from rest_framework import response, mixins, viewsets

from . import renderers

WEBDAV_METHODS = [
    "get",
    "put",
    "lock",
    "unlock",
    "propfind",
    "proppatch",
    "delete",
    "head",
    "options",
    "mkcol",
    "copy",
    "move",
]
ACTIONS = {m: "action_{}".format(m) for m in WEBDAV_METHODS}


class WebDAVViewSet(viewsets.GenericViewSet):
    http_method_names = WEBDAV_METHODS
    renderer_classes = (renderers.WebDAVXMLRenderer,)

    def dispatch(self, request, *args, **kwargs):
        print("BODY", request.body)
        response = super().dispatch(request, *args, **kwargs)
        response["DAV"] = "1"
        return response

    @classmethod
    def as_view(cls, *args, **kwargs):
        actions = {}
        for method in WEBDAV_METHODS:
            handler_name = "action_{}".format(method)
            if hasattr(cls, handler_name):
                actions[method] = handler_name

        kwargs["actions"] = actions
        return super().as_view(*args, **kwargs)


class LibraryViewSet(WebDAVViewSet):
    authentication_classes = []
    permission_classes = []

    def action_propfind(self, request, *args, **kwargs):
        return response.Response({})

    def action_options(self, request, *args, **kwargs):
        return response.Response({})

    def action_get(self, request, *args, **kwargs):
        return response.Response({})

    def action_put(self, request, *args, **kwargs):
        return response.Response({})
